/* eslint-disable @next/next/no-img-element */
import * as React from 'react';
import { useCallback, useState } from 'react';
import { useOverviewImage } from './utils/useOverviewImage';

export const OverviewImagesModal: React.FC = () => {
  const [containerRect, setContainerRect] = useState<DOMRect>();
  const { imageUrl, size, linkConfigs } = useOverviewImage({ containerRect });
  const { width, height } = size;

  const handleRect = useCallback((node: HTMLDivElement | null) => {
    if (!node) {
      return;
    }

    setContainerRect(node.getBoundingClientRect());
  }, []);

  if (!imageUrl) {
    return null;
  }

  return (
    <div
      data-testid="overview-images-modal"
      className="flex h-full w-full items-center justify-center bg-white"
      ref={handleRect}
    >
      <div className="relative" style={{ width, height }}>
        <img alt="overview" className="block h-full w-full" src={imageUrl} />
        {linkConfigs.map(({ size: linkSize, onClick, idObject }) => (
          // eslint-disable-next-line jsx-a11y/click-events-have-key-events
          <div
            role="button"
            tabIndex={0}
            key={idObject}
            style={{
              height: linkSize.height,
              width: linkSize.width,
              top: linkSize.top,
              left: linkSize.left,
              background: 'rgba(0,0,0,0.2)',
              position: 'absolute',
            }}
            onClick={onClick}
          />
        ))}
      </div>
    </div>
  );
};
