import { NOOP } from '@/constants/common';
import { useAppDispatch } from '@/redux/hooks/useAppDispatch';
import { useAppSelector } from '@/redux/hooks/useAppSelector';
import { mapOpenedMapsSelector } from '@/redux/map/map.selectors';
import { openMapAndSetActive, setActiveMap, setMapPosition } from '@/redux/map/map.slice';
import { closeModal, setOverviewImageId } from '@/redux/modal/modal.slice';
import { modelsDataSelector } from '@/redux/models/models.selectors';
import { MapModel, OverviewImageLink } from '@/types/models';
import {
  OverviewImageLinkImageHandler,
  OverviewImageLinkModelHandler,
} from '../OverviewImageModal.types';

interface UseOverviewImageLinkActionsResult {
  handleOnLinkClick(link: OverviewImageLink): void;
}

export const useOvervieImageLinkActions = (): UseOverviewImageLinkActionsResult => {
  const dispatch = useAppDispatch();
  const openedMaps = useAppSelector(mapOpenedMapsSelector);
  const models = useAppSelector(modelsDataSelector);

  const isMapAlreadyOpened = (modelId: number): boolean =>
    openedMaps.some(map => map.modelId === modelId);

  const getModelById = (modelId: number): MapModel | undefined =>
    models.find(map => map.idObject === modelId);

  const onSubmapClick: OverviewImageLinkModelHandler = link => {
    const modelId = link.modelLinkId;
    const model = getModelById(modelId);
    const isMapOpened = isMapAlreadyOpened(modelId);
    if (!model) {
      return;
    }

    if (isMapOpened) {
      dispatch(setActiveMap({ modelId }));
    } else {
      dispatch(openMapAndSetActive({ modelId, modelName: model.name }));
    }

    dispatch(
      setMapPosition({
        x: link.modelPoint.x,
        y: link.modelPoint.y,
        z: link.zoomLevel + model.minZoom,
      }),
    );
    dispatch(closeModal());
  };

  const onImageClick: OverviewImageLinkImageHandler = link => {
    dispatch(setOverviewImageId(link.imageLinkId));
  };

  const handleOnLinkClick: UseOverviewImageLinkActionsResult['handleOnLinkClick'] = link => {
    if ('imageLinkId' in link) {
      return onImageClick(link);
    }

    if ('modelLinkId' in link) {
      return onSubmapClick(link);
    }

    return NOOP;
  };

  return {
    handleOnLinkClick,
  };
};
